/*
   From "Java Precisely", page 59,
   http://www.itu.dk/~sestoft/javaprecisely/javaprecisely-online.pdf

   NOTE: This "bounded buffer" has only one cell!

   Question: Can you turn this into a bounded buffer with two cells?
*/

class BoundedBuffer
{
   private int contents;          // this is our one cell bounded buffer
   private boolean empty = true;  // this is our "condition variable"

   public int get()
   {
      synchronized (this)  // Use a synchronized block
      {
         while (empty)
            try { this.wait(); } catch (InterruptedException x) {};
         empty = true;
         this.notify();
         return contents;
      }
   }

   public void put(int v)
   {
      synchronized (this)  // Use a synchronized block
      {
         while (!empty)
            try { this.wait(); } catch (InterruptedException x) {};
         empty = false;
         contents = v;
         this.notify();
      }
   }
}


class Producer extends Thread
{
   private BoundedBuffer b;
   private int initValue;

   Producer (BoundedBuffer b, int initValue)
   {
      this.b = b;
      this.initValue = initValue;
   }

   public void run()
   {
      for(int i = 0; i < 1000; i++)
      {
         b.put(initValue + i);
      }
   }
}


class Consumer extends Thread
{
   private BoundedBuffer b;

   Consumer (BoundedBuffer b)
   {
      this.b = b;
   }

   public void run()
   {
      for (;;)
      {
         int i = b.get();
         System.out.println(i);
      }
   }
}


public class BoundedBufferTester
{
   public static void main(String[] args)
   {
      BoundedBuffer b = new BoundedBuffer();

      new Producer(b, 0).start();
      new Consumer(b).start();
      //new Producer(b, 2000).start();
   }
}